/*************************************************************
UZI (Unix Z80 Implementation) Sharp MZ-800 common:  mzcommon.c
**************************************************************/


#include "unix.h"
#include "extern.h"
#include <string.h>

/* Screen constants and variables */
#define SCRN_MAX_X 80
#define SCRN_MAX_Y 25
#define SCRN_FONT_ADDR 0c000h - 8 * 256			/* ISO font is stored at the end of the second screen */
#define SCRN_LINE_ADDR_POS SCRN_FONT_ADDR - 25 * 2;	/* Screen line addresses are stored just before it */

static char copyright[] = "MZI Z80 Unix 0.16 for Sharp MZ-8xx, (c) 2003 MZ Community, Roman Dolej\nUpozornn: program nen funkn. Informace na http://mz-800.aktualne.cz/mzi\n";
static uint16 scrn_pos_x = 0;
static uint16 scrn_pos_y = 0;

/**
 * Prints the character on the screen at the cursor position
 * @todo special control processing
 * @todo hardware scrolling implementation
 */
void _putc( char c ) {

  if( c == 10 ) {			/* line feed (in unix we have separate LF and CR) */
    scrn_pos_x = 0;
    scrn_pos_y++;

  } else if( c == 13 ) {		/* carriage return */
    scrn_pos_x = 0;

  } else {
#asm
	ld	l, (ix+6)		; get the argument
	ld	h, 0
	add	hl, hl			; get the source data position
	add	hl, hl
	add	hl, hl
	ld	bc, SCRN_FONT_ADDR
	add	hl, bc
	ex	de, hl

	ld	bc, (_scrn_pos_y)	; get the destination position in video ram
	ld	hl, SCRN_LINE_ADDR_POS
	add	hl, bc
	add	hl, bc
	in	a, (0e0h)
	ld	c, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, c
	ld	bc, (_scrn_pos_x)
	add	hl, bc

	ld	bc, 80
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	ld	a, (de)
	ld	(hl), a
	inc	de
	add	hl, bc
	in	a, (0e1h)
#endasm
    scrn_pos_x++;
  }

  /* Now we have to check the cursor boundaries */
  if( scrn_pos_x == SCRN_MAX_X ) {
    scrn_pos_x = 0;
    scrn_pos_y++;
  }
  if( scrn_pos_y == SCRN_MAX_Y ) {
#asm
	ld	hl, 8000h + 80 * 8	; @todo hardware scrolling
	ld	de, 8000h
	ld	bc, 192 * 80
	in	a, (0e0h)
	ld	a, 1			; we're working with the first page here
	out	(0cdh), a
	ldir
	ld	hl, 8000h + 80 * 8 * 24	; clear last line
	ld	de, 8001h + 80 * 8 * 24
	ld	bc, 8 * 80
	ld	(hl), l
	ldir
	ld	a, 4			; put it back to normal vram read mode 
	out	(0cdh), a		; (when calling a in-vram code, no changes to the access regs are made)
	in	a, (0e1h)
#endasm
    scrn_pos_y--;
  }
}


/* ------------------------------- keyboard handling --------------------------------- */
/* Keyboard routines are based on a polling mechanism. Whenever a keypress is detected */
/* during standard interrupt, the key is added to the queue of yet unread keys.        */
/* System later checks the queue status and gets all queued keystrokes.                */
/* ----------------------------------------------------------------------------------- */

#define KBD_BUFFER_SIZE 20
static char kbd_buffer[ KBD_BUFFER_SIZE ];
static int kbd_pos_write;
static int kbd_pos_read;


/**
 * Updates the keyboard status. This method is called every interrupt
 * and can contain all required keyport scanning.
 * @todo implement the keyboard processing
 */
void mz_process_keyboard() {
}

/**
 * Returns first available keystroke. 
 */
char mz_get_keystroke() {
  char c;

  mz_process_keyboard();

  if( kbd_pos_read == kbd_pos_write )
    return -1;
  c = kbd_buffer[ kbd_pos_read++ ];
  if( kbd_pos_read == KBD_BUFFER_SIZE )
    kbd_pos_read = 0;
  return c;
}

/**
 * Initializes the keyboard tables
 */
void mz_init_keyboard() {
  kbd_pos_read = 0;
  kbd_pos_write = 0;
}


/* ------------------------------------ clock ------------------------------------- */


/**
 * Updates global calendar datetime. This is supposed to be read from a dedicated hardware.
 * However, there's no such hardware on Sharp, thus a clock-based simulation has to be employed.
 * @todo implement the clock
 */
void rdtod() {

  tod.t_time = 0;
  tod.t_date = 0;
/*
  tod.t_time = (tread(SECS)>>1) | (tread(MINS)<<5) | (tread(HRS)<<11);
  tod.t_date = tread(DAY) | (tread(MON)<<5) | (YEAR<<9);
*/
}

/**
 * Stops the running clock counter
 * @todo implement the clock
 */
void mz_stop_clock() {
}

/**
 * Starts the stopped clock counter
 * @todo implement the clock
 */
void mz_start_clock() {
}

/**
 * Checks if the clock counter is elapsed
 * @todo implement the clock
 */
int mz_is_clock_elapsed() {
  return 1;
}

/**
 * Initializes the clock counter
 * @todo implement the clock
 */
void mz_init_clock() {
}


/* ---------------------------- paralel port support ------------------------------ */


/**
 * Prints the character on the paralel port
 * @todo implement the paralel port support
 */
void lpout( char c ) {
}


/* -------------------------------- complete init --------------------------------- */


/**
 * Prints the copyright on the screen
 */
void mz_print_copyright() {

  int i = 0;
  int size = strlen( copyright );
  while( i < size )
    _putc( copyright[ i++ ] );
}


/**
 * Initializes all MZ-specific stuff here
 */
void mz_init() {

  mz_init_keyboard();
  mz_init_clock();
  mz_print_copyright();
}
